﻿using UniversityContracts.BindingModels;
using UniversityContracts.ViewModels;
using UniversityDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace UniversityDatabaseImplement.Models
{
    public class Statement : IStatementModel
    {
        public int Id { get; set; }
        [Required]
        public DateTime Date { get; set; }
        [Required]
        public int HoursCount { get; set; }
        [ForeignKey("StatementId")]
        public virtual List<StatementStudent> StatementStudents { get; set; } = new();
        [ForeignKey("StatementId")]
        public virtual List<ExaminationResult> ExaminationResults { get; set; } = new();
        private Dictionary<int, IStudentModel>? _students = null;
        public Dictionary<int, IStudentModel> Students
        {
            get
            {
                if(_students == null)
                {
                    _students = StatementStudents.ToDictionary(
                        x => x.Student.Id, x => x.Student as IStudentModel);
                }

                return _students;
            }
        }
        public static Statement Create(StatementBindingModel model)
        {
            return new Statement
            {
                Id = model.Id,
                Date = model.Date,
                HoursCount = model.HoursCount
            };
        }
        public void Update(StatementBindingModel model)
        {
            if (model == null) return;

            Id = model.Id;
            Date = model.Date;
            HoursCount = model.HoursCount;
        }

        public StatementViewModel GetViewModel => new()
        {
            Id = Id,
            Date = Date,
            HoursCount = HoursCount,
            Students = Students
        };

    }
}
