﻿using UniversityContracts.BindingModels;
using UniversityContracts.BuisnessLogicContracts;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace UniversityBuisnessLogic.BuisnessLogic
{
    public class ReportTypeLogic : IReportTypeLogic
    {
        private readonly ILogger _logger;
        private readonly IReportTypeStorage _reportTypeStorage;

        public ReportTypeLogic(ILogger<ReportTypeLogic> logger, IReportTypeStorage reportTypeStorage)
        {
            _logger = logger;
            _reportTypeStorage = reportTypeStorage;
        }

        public bool Create(ReportTypeBindingModel model)
        {
            CheckModel(model);
            if (_reportTypeStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(ReportTypeBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_reportTypeStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public ReportTypeViewModel? ReadElement(ReportTypeSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ReportTypeName: {Name}. Id:{Id}", model.Name, model.Id);
            var element = _reportTypeStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<ReportTypeViewModel>? ReadList(ReportTypeSearchModel? model)
        {
            _logger.LogInformation("ReadList. ReportTypeName: {Name}. Id:{Id}", model?.Name, model?.Id);
            var list = model == null ? _reportTypeStorage.GetFullList() : _reportTypeStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(ReportTypeBindingModel model)
        {
            CheckModel(model);
            if (_reportTypeStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(ReportTypeBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия типа отчета", nameof(model.Name));
            }

            _logger.LogInformation("ReportType. ReportName:{Name}. Id: { Id}", model.Name, model.Id);
            var element = _reportTypeStorage.GetElement(new ReportTypeSearchModel
            {
                Name = model.Name,
            });

            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Тип отчета с таким названием уже есть");
            }
        }
    }
}
