﻿using UniversityContracts.BindingModels;
using UniversityContracts.BuisnessLogicContracts;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace UniversityBuisnessLogic.BuisnessLogic
{
    public class StudentLogic : IStudentLogic
    {
        private readonly ILogger _logger;
        private readonly IStudentStorage _studentStorage;

        public StudentLogic(ILogger<StudentLogic> logger, IStudentStorage studentStorage)
        {
            _logger = logger;
            _studentStorage = studentStorage;
        }
        public bool Create(StudentBindingModel model)
        {
            CheckModel(model);
            if (_studentStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(StudentBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_studentStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public StudentViewModel? ReadElement(StudentSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. StudentName: {Name}. Id:{Id}", model.Name, model.Id);
            var element = _studentStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public List<StudentViewModel>? ReadList(StudentSearchModel? model)
        {
            _logger.LogInformation("ReadList. StudentName: {Name}. Id:{Id}", model?.Name, model?.Id);
            var list = model == null ? _studentStorage.GetFullList() : _studentStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool Update(StudentBindingModel model)
        {
            CheckModel(model);
            if (_studentStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(StudentBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет имени студента", nameof(model.Name));
            }

            _logger.LogInformation("Student. Name:{Name}. Id: { Id}", model.Name, model.Id);
        }
    }
}
